@@ -65,19 +65,25 @@ if(FALSE)
 ## A few ones have a very simple method:
 ## Note that the 'sign' slot is from the C-internal struct
 ## and is always +/- 1 , but R's sign(0) |--> 0
+.getSign <- function(x) vapply(getD(x), slot, 1L, "sign")
 .mpfr.sign <- function(x) {
     r <- numeric(n <- length(x))# all 0
     not0 <- !mpfr.is.0(x)
-    r[not0] <- unlist(lapply(x@.Data[not0], slot, name = "sign"))
+    r[not0] <- .getSign(x[not0])
     r
 }
 setMethod("sign", "mpfr", .mpfr.sign)
 
+## R version, no longer used:
+.abs.mpfr <- function(x) {
+    ## FIXME: faster if this happened in a .Call
+    xD <- getDataPart(x)   # << currently [2011] *faster* than  x@Data
+    for(i in seq_along(x))
+        slot(xD[[i]], "sign", check=FALSE) <- 1L
+    setDataPart(x, xD, check=FALSE) ## faster than  x@.Data <- xD
+}
 setMethod("abs", "mpfr",
-	  function(x) {
-	      for(i in seq_along(x)) x[[i]]@sign <- 1L
-	      x
-	  })
+	  function(x) .Call(Rmpfr_abs, x))
 
 ## Note that  factorial() and lfactorial() automagically work through  [l]gamma()
 ## but for the sake of "exact for integer"
@@ -91,7 +97,7 @@ setMethod("factorial", "mpfr",
 ## The "real" thing is to use  the MPFR-internal function:
 factorialMpfr <- function(n, precBits = max(2, ceiling(lgamma(n+1)/log(2)))) {
     stopifnot(n >= 0)
-    new("mpfr", .Call("R_mpfr_fac", n, precBits, PACKAGE="Rmpfr"))
+    new("mpfr", .Call(R_mpfr_fac, n, precBits))
 }
 
 ##' Pochhammer rising factorial = Pochhammer(a,n) {1 of 2 definitions!}
@@ -105,8 +111,9 @@ pochMpfr <- function(a, n) {
         a <- mpfr(a, precBits = pmax(1,n)*getPrec(a))
     else if((ln <- length(n)) != 1 && ln != length(a))
 	a <- a + 0*n
-    a@.Data[] <- .Call("R_mpfr_poch", a, n, PACKAGE="Rmpfr")
-    a
+    ## a@.Data[] <- .Call(R_mpfr_poch, a, n)
+    ## a
+    setDataPart(a, .Call(R_mpfr_poch, a, n))
 }
 
 ##' Binomial Coefficient choose(a,n)
@@ -120,8 +127,9 @@ chooseMpfr <- function(a, n) {
         a <- mpfr(a, precBits = n + max(2, precB))
     } else if((ln <- length(n)) != 1 && ln != length(a))
 	a <- a + 0*n
-    a@.Data[] <- .Call("R_mpfr_choose", a, n, PACKAGE="Rmpfr")
-    a
+    ## a@.Data[] <- .Call(R_mpfr_choose, a, n)
+    ## a
+    setDataPart(a, .Call(R_mpfr_choose, a, n))
 }
 
 chooseMpfr.all <- function(n) { ## return   chooseMpfr(n, 1:n)  "but smartly"
@@ -149,8 +157,9 @@ chooseMpfr.all <- function(n) { ## return   chooseMpfr(n, 1:n)  "but smartly"
 ##' @author Martin Maechler
 roundMpfr <- function(x, precBits) {
     stopifnot(is(x, "mpfr"))
-    x@.Data[] <- .Call("R_mpfr_round", x, precBits, PACKAGE="Rmpfr")
-    x
+    ## x@.Data[] <- .Call(R_mpfr_round, x, precBits)
+    ## x
+    setDataPart(x, .Call(R_mpfr_round, x, precBits))
 }
 
 ## "log" is still special with its 'base' :
@@ -158,17 +167,11 @@ setMethod("log", signature(x = "mpfr"),
 	  function(x, base) {
 	      if(!missing(base) && base != exp(1))
 		  stop("base != exp(1) is not yet implemented")
-	      x@.Data[] <- .Call("Math_mpfr", x, .Math.codes[["log"]],
-				 PACKAGE="Rmpfr")
-	      x
+	      setDataPart(x, .Call(Math_mpfr, x, .Math.codes[["log"]]))
 	  })
 
-setMethod("Math", signature(x = "mpfr"),
-	  function(x) {
-	      x@.Data[] <- .Call("Math_mpfr", x, .Math.codes[[.Generic]],
-			   PACKAGE="Rmpfr")
-	      x
-	  })
+setMethod("Math", signature(x = "mpfr"), function(x)
+	  setDataPart(x, .Call(Math_mpfr, x, .Math.codes[[.Generic]])))
 
 setMethod("Math2", signature(x = "mpfr"),
 	  function(x, digits) {
